package calc.implementation;

import calc.CalcRemoteInterface;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.ExportException;
import java.rmi.server.UnicastRemoteObject;


public class CalcServer {
    
    //the default port for rmiregistry is 1099;
    //instead, by default connections to remote objects are on random ports
    //(otherwise specified in the export command)
    private static final int REGISTRY_PORT=1099;
    //the "myRegistry" reference variable isn't actually used; 
    //for the sake of clarity, we just assign it the value returned 
    //by the registry creation method.
    private static Registry myRegistry;
    private static final String REGISTRY_HOST="127.0.0.1";
    private static String SERVANT_NAME;

    public CalcServer(){
        this("Calculator"); //default name
    }
    public CalcServer(String servantName) {
        SERVANT_NAME = servantName;
        try {
            //a servant object is created
            CalcRemoteInterface c = new CalcRemoteObj();
//            //next: binding to the rmiregistry
            Naming.rebind("//"+REGISTRY_HOST+":"+Integer.toString(REGISTRY_PORT)+"/"+SERVANT_NAME, c);
            //Naming.rebind("//"+REGISTRY_HOST+":"+Integer.toString(REGISTRY_PORT)+"/"+"Mickey", c);

            
            //alternative way, which is mandatory in case CalcRemoteObject 
            //would not extend UnicastRemoteObject, as it actually does.
//            try {
//                CalcRemoteInterface calcStub = 
//                        (CalcRemoteInterface)UnicastRemoteObject.exportObject(c, 0); // 0 means: default
//                myRegistry.rebind(SERVANT_NAME, calcStub);
//            } catch (ExportException e) {
//                System.err.println("Outch! Servant \""+SERVANT_NAME+"\" has already been exported!\n");
//                myRegistry.rebind(SERVANT_NAME, c);
//            }
            
        } catch (Exception e) { 
            System.err.println("Trouble: " + e);
            e.printStackTrace();
        }
    }
  
    public static void startRegistry(int port) throws RemoteException{

        myRegistry = LocateRegistry.createRegistry(REGISTRY_PORT);
        System.out.println("Registry created - port "+Integer.toString(REGISTRY_PORT));
        
        //be careful in using LocateRegistry.createRegistry(..):
        //such a method returns a local reference, but no connection to the 
        //registry is actually created. Thus, in case of problems, they will
        //appear later on.
    }

    
    public static void main(String[] args) throws Exception {
        
        // Remember:
        // RMI's class loader will download classes from remote locations 
        // only if a security manager has been set.
        if (System.getSecurityManager() == null) {
            // java.lang.SecurityManager 
            System.setSecurityManager(new SecurityManager());
        }
        
        startRegistry(REGISTRY_PORT);
                       
        //the server is set up
        new CalcServer();
        
        // QUESTION: Why the server keeps running, 
        // even if it reaches this "terminal" point?
        
    }
}
